# frozen_string_literal: true

require 'faraday'
require 'json'

require 'yousty/http/request_method'

module Yousty
  module HTTP
    # Sends actual requests to the Gallery API server
    #
    class Connection
      DEFAULT_HEADERS = {
        'Content-Type' => 'application/vnd.api+json'
      }.freeze

      # @param [Symbol] - A name of HTTP method to be sent: [:get, :patch, :put, :post, :delete]
      # @param [String] - A path to the called endpoint
      # @param body [Hash] - Request body to be sent to the server
      # @param headers [Hash] - Custom request headers to be merged into the default ones
      # @return Faraday::Response
      #
      def call(method_name, path, body: {}, headers: {})
        request_method = RequestMethod.new(method_name)
        connection.method(request_method.to_s).call(path) do |req|
          req.headers = req.headers.merge(headers)
          req.body = JSON.generate(body)
        end
      end

      def authorize!(auth_token)
        @connection.authorization :Bearer, auth_token
      end

      private

      attr_reader :connection

      def initialize(host, auth_token: nil, connection: nil)
        @connection = connection || Faraday.new(url: host, headers: DEFAULT_HEADERS)
        authorize!(auth_token) if auth_token
      end
    end
  end
end
